'use strict';

var _ = require('lodash');
var Type = require('./type');
var SchemaUrl = 'http://json-schema.org/draft-04/schema#';

function processArray(array) {
  var arraySchema = {
    "items": {}
  };
  _.forEach(array, function(value, key) {
    //process every array element
    var type = Type.typeIs(value);
    var elementSchema = {};

    if (type === "object") {
      elementSchema = processObject(value);
    } else if (type === "array") {
      elementSchema = processArray(value);
    } else {
      elementSchema = {
        "type": type
      }
    }
    arraySchema.items = elementSchema;
  });

  return arraySchema;
};

function processObject(object) {
  var objectSchema = {
    "properties": {},
    "required": []
  };
  _.forIn(object, function(value, key) {
    var type = Type.typeIs(value);
    var elementSchema = {};
    if (type === "object") {
      elementSchema = processObject(value);
    } else if (type === "array") {
      elementSchema = processArray(value);
    } else {
      elementSchema = {
        "type": type
      };
    }

    objectSchema.properties[key] = elementSchema;
    objectSchema.required = _.keys(object);
  });
  return objectSchema;
};


var processJson = function(title, json) {
  var jsonSchema = {};
  var type = Type.typeIs(json);
  switch (type) {
    case "object":
      var objectSchema = processObject(json);
      jsonSchema = objectSchema;
      jsonSchema.type = "object";
      break;
    case "array":
      //set type & add items property for array type
      var arraySchema = processArray(json);
      jsonSchema = arraySchema;
      jsonSchema.type = "array";
      break;
    default:
  }
  jsonSchema['$schema'] = SchemaUrl;
  jsonSchema.title = title;
  return jsonSchema;
};

module.exports = processJson;
