'use strict';

var _ = require('lodash');
var fs = require('fs');
var Type = require('./type');
var jsonSchema = require('./jsonSchema');

var SchemaTemplatePath = __dirname + "/mongooseSchemaTemplate.js";

//mongooseSchema
var processJson = function(title, json, storePath, callback) {
  var theSchema = generateSchema(title, json);
  //replace double quotation and format the file to generate pure mongoose schema
  var pureSchema = JSON.stringify(theSchema)
    .replace(/"/g, "")
    .replace(/,/g, ", \n   ")
    .replace(/{/g, " { \n   ")
    .replace(/}/g, "\n   }");

  fs.readFile(SchemaTemplatePath, 'utf8', function(err, data) {
    if (err) {
      return callback(err);
      // return console.log(err);
    }
    var targetFile = data
      .replace(/schemaDefinition/g, pureSchema)
      .replace(/MongooseSchema/g, title + "Schema")
      .replace(/SchemaName/g, _.upperFirst(title));
    fs.writeFile(storePath, targetFile, 'utf8', function(err) {
      if (err) {
        return callback(err);
        // return console.log(err);
      }
      callback(null, theSchema);
      return console.log(title + " has been stored to " + storePath);
    });
  });
};

function generateSchema(title, json) {
  var schema = jsonSchema(title, json);
  var schemaProperties = {};
  if (schema.type === "array") {
    schemaProperties = schema.items.properties;
  } else if (schema.type === "object") {
    schemaProperties = schema.properties;
  } else {
    console.log(" for further type ... ");
  }
  var schemaObject = processSchema(schemaProperties);
  return schemaObject;
};

function generateSchemaFromJsonSchema(title, jsonSchema) {
  var schemaProperties = {};
  if (jsonSchema.type === "array") {
    schemaProperties = schema.items.properties;
  } else if (jsonSchema.type === "object") {
    schemaProperties = jsonSchema.properties;
  } else {
    console.log(" for further type ... ");
  }
  var schemaObject = processSchema(schemaProperties);
  return schemaObject;
};

function processSchema(schemaProperties) {
  var schemaObject = {};

  _.forIn(schemaProperties, function(value, key) {
    if (value.items) {

      if (value.items.properties) {
        schemaObject[key] = [processSchema(value.items.properties)];
      } else {
        schemaObject[key] = [_.upperFirst(value.items.type)];
      }
    } else if (value.properties) {
      schemaObject[key] = processSchema(value.properties);
    } else {
      schemaObject[key] = _.upperFirst(value.type);
    }
  });

  return schemaObject;
};

exports.mongooseSchema = processJson;
exports.mongooseSchemaFromJsonSchema = generateSchemaFromJsonSchema;